/*
 *  FindingVisitor.h
 *  testingTinyXMLandCollada
 *
 *  Created by Matt Evett on 4/18/12.
 *  Copyright 2012 Eastern Michigan Univ. All rights reserved.
 *
 *  Purpose of the class is to find the text node held by the first DOM element
 * visited with the given name.
 *
 */

#ifndef FINDINGVISITOR_INCLUDED
#define FINDINGVISITOR_INCLUDED

#include "tinyxml.h"

class FindingVisitor: public TiXmlVisitor {
private:
	const char* targetName;  //Name of the element we seek.
	const char* preTargetName; //Name of element in whose context the target will be
	bool elementFound_p;
	bool preElementFound_p;
	const char *valueOfFoundElement;  //Value of the found element's text element

public:
	
	FindingVisitor(const char* nameOfElem, const char* nameOfPreElem = NULL) :
		targetName(nameOfElem),
		preTargetName(nameOfPreElem),
		preElementFound_p(false),
		elementFound_p(false)
		{}
	
	bool foundTheElement() {
		return elementFound_p;
	}
	
	const char *value() {
		return valueOfFoundElement;
	}

	/// Visit an element.
	/*virtual*/ bool VisitEnter( const TiXmlElement& element, const TiXmlAttribute* firstAttribute );	/// Visit an element.

};




#endif // FINDINGVISITOR_INCLUDED