/*
 *  SketchUpCollada.cpp
 *  testingTinyXMLandCollada
 *
 *  Created by Matt Evett on 4/23/12.
 *  Copyright 2012 Eastern Michigan Univ. All rights reserved.
 *
 */

#include "SketchUpCollada.h"

SketchUpCollada::SketchUpCollada(const char* nameOfColladaFile) : colladaFileName(nameOfColladaFile), doc(nameOfColladaFile) {
	isBound = doc.LoadFile();
	if (isBound) {
		isBound = getVertexCoordinates() && getTriangleDefinitions() && getEdgeDefinitions();
	}
	else
		fprintf(stderr, "TinyXML failed to load file \"%s\", probably because there is no file with that pathname.\n", nameOfColladaFile);
}

bool SketchUpCollada::isLoadedFile() {
	return isBound;
}

bool SketchUpCollada::getElemValue(std::vector<float> &store, const char* elemName, const char* elemContextName) {
	bool result;
	
	FindingVisitor finder(elemName, elemContextName);
	doc.Accept(&finder);  //Traverse the xml, looking for the named element and its value
	
	result = finder.foundTheElement();
	if (result) {
		std::string data = finder.value();
		
		std::stringstream   dataStream(data);  //Convert the string of numbers into a vector of floats
		std::copy(std::istream_iterator<float>(dataStream),
				  std::istream_iterator<float>(),
				  std::back_inserter(store)
				  );					
	}
	else
		fprintf(stderr, "\nSketchUpCollada could not find an element named %s.\n",elemName);
	
	return result;
}


bool SketchUpCollada::getVertexCoordinates() {
	bool result = getElemValue(coordinates, "float_array" );
	if (!result)
		fprintf(stderr,"SketchUpCollada unable to find vertext coordinate information in %s.",colladaFileName);
	return result;
}


bool SketchUpCollada::getTriangleDefinitions() {
	bool result;
	std::vector<float> pointIndicesDefiningTrianglesAsFloats;  // Series of indices into coordinate array naming vertices defining triangles
	result = getElemValue(pointIndicesDefiningTrianglesAsFloats, "p", "triangles" ); //Watch for data leak here
	if (result) {	
		for(int i=0;i< pointIndicesDefiningTrianglesAsFloats.size();i++)
			pointsDefiningTriangles.push_back(( int)(pointIndicesDefiningTrianglesAsFloats[i]));
	}
	else
		fprintf(stderr,"SketchUpCollada unable to find triangle definitions in %s.",colladaFileName);
		
	return result;
}


bool SketchUpCollada::getEdgeDefinitions() {
	bool result;
	std::vector<float> pointIndicesDefiningEdgesAsFloats;  // Series of indices into coordinate array naming vertices defining edges
	result = getElemValue( pointIndicesDefiningEdgesAsFloats, "p", "lines" ); //Watch for data leak here
	
	if (result)
		for(int i=0;i< pointIndicesDefiningEdgesAsFloats.size();i++)
			pointsDefiningEdges.push_back(( int)(pointIndicesDefiningEdgesAsFloats[i]));
	else
		fprintf(stderr,"SketchUpCollada unable to find triangle definitions in %s.",colladaFileName);
	return result;
}
	
std::vector<float> SketchUpCollada::vertexCoordinates() {
	return coordinates;
}
std::vector<int> SketchUpCollada::triangleDefinitions() {
	return pointsDefiningTriangles;
}	
std::vector<int> SketchUpCollada::edgeDefinitions() {
	return pointsDefiningEdges;
}

