/*
 *  SketchUpCollada.h
 *  testingTinyXMLandCollada
 *
 *  Created by Matt Evett on 4/23/12.
 *  Copyright 2012 Eastern Michigan Univ. All rights reserved.
 *  
 *  This class allows access to some of the internal data of a Google SketchUp 
 *  collada export file.  (The name of such a file typically has a ".dae"
 *  suffix.
 */

#ifndef SketchUpCollada_included
#define SketchUpCollada_included

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <iterator>
#include <algorithm>
//Next line is for Windows/Visual C. Uncomment it to speed compilation there.
//#include "stdafx.h"
#include "tinyxml.h"
#include "FindingVisitor.h"


class SketchUpCollada {
private:
	TiXmlDocument doc;
	bool isBound;
	SketchUpCollada() : isBound(false) {} // Not for public use
	const char *colladaFileName;

	bool getElemValue(std::vector<float> & /* result here*/, const char* /*elemName*/, const char* /* *elemContextName*/ = NULL);
	std::vector<float> coordinates;  // x,y,z coordinates of points defining the shape
	std::vector<int> pointsDefiningTriangles; // Series of indices into coordinate vector naming vertices defining triangles
	std::vector<int> pointsDefiningEdges;  // Series of indices into coordinate vector naming vertices defining edges

	bool getVertexCoordinates();
	
	bool getTriangleDefinitions();
	
	bool getEdgeDefinitions();

public:
	/* 
	 Currently the only legal constructor so that all SketchUpColladas are bound to a file.
	 */
	SketchUpCollada(const char* /*nameOfColladaFile*/);
		
	/* Return true if we successfully found and parsed a collada file */
	bool isLoadedFile();
	
	/*
	 Returns a vector of floats consisting of the coordinates of the object defined by the
	 collada file.  The vector elements will be of the form:
	 x0, y0, z0, x1, y1, z1, x2, y2, z2, etc.  Thus the first three elements are the coordinates
	 of vertex 0, the next three are the coordinates of vertex 1, etc.
	 There will appear to be more vertices than needed because the collada file contains points
	 duplicating the vertices to define the "back sides" of each triangle, as well as visible edges.
	 */
	std::vector<float> vertexCoordinates();
	
	/*
	 Returns a vector of ints that are indices of the points defining the triangles comprising
	 the faces of the object defined in the collada file.
	 The indices correspond to the vector returned by getVertexCoordinates.  Thus a '1' here refers
	 to point 1, the coordinates of which are(x1, y1, z1), as defined in the other function (i.e.,
	 the 4th, 5th and 6th elements of the vector returned by getVertexCoordinates.)
	 */
	
	std::vector<int> triangleDefinitions();
	
	/*
	 Returns a vector of ints that are indices of the points defining the edges of the object defined 
	 in the collada file.
	 The indices correspond to the vector returned by getVertexCoordinates.  Thus a '1' here refers
	 to point 1, the coordinates of which are(x1, y1, z1), as defined in the other function (i.e.,
	 the 4th, 5th and 6th elements of the vector returned by getVertexCoordinates.)
	 */
	
	std::vector<int> edgeDefinitions();
};
#endif	
